from typing import Literal

ChatModel = Literal[
    "qwen3-max-preview",
    "qwen-plus-2025-09-11",
    "qwen3-235b-a22b",
    "qwen3-coder-plus",
    "qwen3-30b-a3b",
    "qwen3-coder-30b-a3b-instruct",
    "qwen3-32b",
    "qwen-max-latest",
    "qwen-plus-2025-01-25",
    "qwen-turbo-2025-02-11",
    "qwq-32b",
    "qvq-72b-preview-0310",
    "qwen2.5-omni-7b",
    "qwen2.5-vl-32b-instruct",
    "qwen2.5-14b-instruct-1m",
    "qwen2.5-coder-32b-instruct",
    "qwen2.5-72b-instruct",
]
